@@ -1,12 +1,13 @@
 from __future__ import absolute_import
 
 import argparse
+
 from flask_script import Command
 
 
 class TaskTigerCommand(Command):
     capture_all_args = True
-    help = 'Run a TaskTiger worker'
+    help = "Run a TaskTiger worker"
 
     def __init__(self, tiger):
         super(TaskTigerCommand, self).__init__()
@@ -15,8 +16,8 @@ class TaskTigerCommand(Command):
     def create_parser(self, *args, **kwargs):
         # Override the default parser so we can pass all arguments to the
         # TaskTiger parser.
-        func_stack = kwargs.pop('func_stack', ())
-        parent = kwargs.pop('parent', None)
+        func_stack = kwargs.pop("func_stack", ())
+        parent = kwargs.pop("parent", None)
         parser = argparse.ArgumentParser(*args, add_help=False, **kwargs)
         parser.set_defaults(func_stack=func_stack + (self,))
         self.parser = parser
