@@ -5,12 +5,13 @@ from __future__ import (
     print_function,
     unicode_literals,
 )
+
 import signal
 
 from .exceptions import JobTimeoutException
 
 
-class BaseDeathPenalty(object):
+class BaseDeathPenalty:
     """Base class to setup job timeouts."""
 
     def __init__(self, timeout):
@@ -45,8 +46,8 @@ class BaseDeathPenalty(object):
 class UnixSignalDeathPenalty(BaseDeathPenalty):
     def handle_death_penalty(self, signum, frame):
         raise JobTimeoutException(
-            'Job exceeded maximum timeout '
-            'value (%d seconds).' % self._timeout
+            "Job exceeded maximum timeout "
+            "value (%d seconds)." % self._timeout
         )
 
     def setup_death_penalty(self):
