@@ -4,15 +4,16 @@ import logging
 import tempfile
 
 from tasktiger import TaskTiger, Worker
-from tests.utils import setup_structlog, get_redis, TEST_TIGER_CONFIG
+
+from tests.utils import TEST_TIGER_CONFIG, get_redis, setup_structlog
 
 tiger = TaskTiger(lazy_init=True)
 
 
 @tiger.task
 def lazy_task(filename):
-    with open(filename, 'w') as f:
-        f.write('ok')
+    with open(filename, "w") as f:
+        f.write("ok")
 
 
 def test_lazy_init():
@@ -22,4 +23,4 @@ def test_lazy_init():
     with tempfile.NamedTemporaryFile() as f:
         lazy_task.delay(f.name)
         Worker(tiger).run(once=True)
-        assert f.read().decode('utf8') == 'ok'
+        assert f.read().decode("utf8") == "ok"
